// Protocol Buffers - Google's data interchange format
// Copyright 2026 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package pb;

option csharp_namespace = "Google.Protobuf.Reflection";
option java_multiple_files = true;
option java_outer_classname = "CSharpFeaturesOuterClass";

import "google/protobuf/descriptor.proto";

extend google.protobuf.FeatureSet {
  optional CSharpFeatures csharp = 1004;
}

message CSharpFeatures {
}
